/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id:
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.ftp;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;

/**
 * Pool of FTP connections
 *
 * @author alouis
 *
 */
public class FTPClientPool {

    private static Map<String, FTPClient> clients = new HashMap<String, FTPClient>();

    /**
     * Return an FTPClient. FTPClients are stored in a Map,which key is
     * <code>server+port+login+password</code>. If a client retrieved from
     * the Map is disconnected, another one is created and connected.
     *
     * @param server
     * @param port
     * @param login
     * @param password
     * @return a connected FTPClient
     * @throws IOException
     */
    public static FTPClient getConnectedFTPClient(final String server, final int port,
            final String login, final String password) throws IOException {
        String key = server + port + login + password;
        FTPClient client = clients.get(key);
        if (client == null || !client.isConnected()) {
            client = createNewFTPClient(server, port, login, password);
            clients.put(key, client);
        }
        return client;
    }

    /**
     * Utility method used to connect to a ftp server.
     *
     * @param server
     *            the server
     * @param port
     *            the port
     * @param login
     *            the login
     * @param password
     *            the password
     * @return the FTP Client
     * @throws IOException
     *             impossible to create a FTP Client
     */
    public static FTPClient createNewFTPClient(final String server, final int port,
            final String login, final String password) throws IOException {
        FTPClient client = new FTPClient();

        client.connect(server, port);
        int reply = client.getReplyCode();

        if(!FTPReply.isPositiveCompletion(reply)) {
            client.disconnect();
         throw new IOException("connection to FTP server failed.");
        }

        boolean connected = client.login(login, password);
        if(!connected){
            throw new IOException("authentication failed.");
        }
        return client;
    }
}
