/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.bc.ftp.connection;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.logging.Logger;

import javax.activation.DataHandler;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.ow2.petals.bc.ftp.FTPClientPool;
import org.ow2.petals.bc.ftp.StreamDataSource;
import org.ow2.petals.component.framework.util.FileNamePatternUtil;
import org.ow2.petals.component.framework.util.FileNamePatternUtil.RegexFileNameFilter;

/**
 * Some IO utilities for the file transfer BC.
 * 
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.2
 * @author alouis
 * 
 */
public final class FTPConnection {

    private Logger logger;

    private FTPConnectionInfo info;

    public FTPConnection(FTPConnectionInfo info, Logger logger) throws IOException {
        super();
        this.info = info;
        this.logger = Logger.getLogger(logger.getName() + ".FTPConnection");
    }

    
    
    /**
     * Put file on the FT server
     * @param attachments  
     * @return
     * @throws IOException
     */
    public boolean mput(HashMap<String, DataHandler> attachments) throws IOException  {

        FTPClient client = null;
        boolean transmit = false;
        boolean isFirstTry = true;
        int numberOfTries = info.getAttempt();
        int transmitedFiles = 0;
        Set<String> lstKey = attachments.keySet();
        String[] fileNames =  new String[lstKey.size()];
        lstKey.toArray(fileNames);
        
        while (isFirstTry || ((transmitedFiles < fileNames.length) && numberOfTries > 0)) {
            
            try {
                if (!isFirstTry)
                    Thread.sleep(info.getDelay());
                isFirstTry = false;
                numberOfTries --;
                client = connectToFTPDirectory();
                for (int i = transmitedFiles; i < fileNames.length; i++) {
                    String fileName = fileNames[i];
                    logger.finest("start sending " + fileName + " file on the FTP server");
                    InputStream stream = attachments.get(fileName).getInputStream();
                    transmit = client.storeFile(fileName, stream);
                    if (transmit) 
                        transmitedFiles ++;
                    stream.close();
                }
            } catch (IOException e) {
                // Problem occurred while sending the current file.
            } catch (InterruptedException e) {
            }
        }

        if (client != null) {
            logger.finest("disconnect FTP connection to " + info.getServer());
            client.logout();
            client.disconnect();
        }
        
        if (transmitedFiles < fileNames.length) {
            throw new IOException("problem while sending " + fileNames[transmitedFiles]
                    + " on the FTP server");
        }
        return true;

    }
        
    
    /**
     * 
     * @param fileName
     * @param content
     * @return
     * @throws IOException
     */
    public  boolean putString(final String fileName, String content) throws IOException {
        FTPClient client = null;
        boolean transmit = false;
        boolean isFirstTry = true;
        int numberOfTries = info.getAttempt();

        while (isFirstTry || (!transmit && numberOfTries > 0)) {

            try {
                if (!isFirstTry)
                    Thread.sleep(info.getDelay());
                isFirstTry = false;
                numberOfTries --;
                client = connectToFTPDirectory();
                logger.finest("start sending " + fileName + " file on the FTP server");
                OutputStream ops = client.storeFileStream(fileName);
                ops.write(content.getBytes());
                ops.flush();
                ops.close();
                transmit = client.completePendingCommand();
                if (transmit)
                    logger.finest(fileName + " sent on the FTP server");
            } catch (IOException ioe) {
            } catch (InterruptedException e) {
            } 

        }

        if (client != null) {
            logger.finest("disconnect FTP connection to " + info.getServer());
            client.logout();
            client.disconnect();
        }
        if (!transmit)
            throw new IOException("Error occurred while storing the " + 
                    fileName + " file on the FTP server");


        return true;

    }

    /**
     * Gets a file from a ftp server.
     * 
     * @param server
     *            the server
     * @param port
     *            the port
     * @param login
     *            the login
     * @param password
     *            the password
     * @param remoteFile
     *            the remote file
     * 
     * @return String
     * @throws FTPBCException
     *             impossible to get: The file does not exist
     */
    public  DataHandler get(final String remoteFile) throws IOException {
        DataHandler result = null;
        FTPClient client = null;
        boolean isFirstTry = true;
        int numberOfTries = info.getAttempt();

        while (isFirstTry || (result == null && numberOfTries > 0)) {
            try {
                if (!isFirstTry)
                    Thread.sleep(info.getDelay());
                isFirstTry = false;
                numberOfTries --;
                client = connectToFTPDirectory();
                FTPFile[] files = client.listFiles(remoteFile);
                if (files != null && files.length > 0) {
                    result = get(files[0], client);
                }
            } catch (IOException e) {
            } catch (InterruptedException e) {
            } 
        }

        if (client != null) {
            logger.finest("disconnect FTP connection to " + info.getServer());
            client.logout();
            client.disconnect();
        }
        
        if (result == null)
            throw new IOException(" Can't get files from the FTP server");

        return result;
    }

   
   /**
    * This method will only get files, it does not include subpath
    * @param fileFilterList
    * @return
    * @throws IOException
    */
    public Map<String, DataHandler> mGet(final List<String> fileFilterList) throws IOException {


        FTPClient client = null;
        boolean isFirstTry = true;
        boolean getAllFiles = false;
        int numberOfTries = info.getAttempt();
        Map<String, DataHandler> datahandlers = null;
        FTPFile[] files = null;
        
        //Construct the fileFilters to match files read from the FTP directory
        List<FileNamePatternUtil.RegexFileNameFilter> filterList = new LinkedList<FileNamePatternUtil.RegexFileNameFilter>();
        for (String string : fileFilterList) {
            filterList.add(FileNamePatternUtil.getInstance()
                    .buildFileNameFilterFromWildChar(string));
        }
        RegexFileNameFilter filter = FileNamePatternUtil.getInstance()
        .buildFileNameFilterFromFilters(filterList);

        while (isFirstTry || (!getAllFiles && numberOfTries > 0)) {
            
            try {
                if (!isFirstTry)
                    Thread.sleep(info.getDelay());
                isFirstTry = false;
                numberOfTries --;
                datahandlers = new HashMap<String, DataHandler>();
                
                if (client == null || !client.isConnected())
                    client = connectToFTPDirectory();

                files = client.listFiles();
                for (FTPFile file : files) {
                    if (file.isFile() && filter.accept(null, file.getName())) {
                        datahandlers.put(file.getName(), get(file, client));
                    }
                }
                getAllFiles = true;
            } catch (IOException e) {

            } catch (InterruptedException e) {

            } 
        }
        
        if (!getAllFiles)
            throw new IOException("Can't get files from the FTP server");



        if (client != null) {
            logger.finest("disconnect FTP connection to " + info.getServer());
            client.logout();
            client.disconnect();
        }


        return datahandlers;
    }

    /**
     * The ftp-client MUST be connected. This method DOES NOT close the
     * ftp-client
     * 
     * @param remoteFile
     * @param client
     *            a ftpclient, MUST be CONNECTED
     * @return the retieved file, as a datahandler
     * @throws IOException
     *             the file is not found or FTP access problem
     */
    public DataHandler get(FTPFile remoteFile, FTPClient client) throws IOException {
        DataHandler result;
        logger.finest("start receiving file " + remoteFile.getName() + " from FTP server");
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream((int) remoteFile.getSize());
        client.retrieveFile(remoteFile.getName(), outputStream);
        outputStream.flush();
        logger.finest(remoteFile.getName() + " file received from FTP server");
        ByteArrayInputStream in = new ByteArrayInputStream(outputStream.toByteArray());
        StreamDataSource datasource = new StreamDataSource(in, remoteFile.getName());
        result = new DataHandler(datasource);
        logger.finest(remoteFile.getName() + " set as Stream");
        return result;
    }

    
    /**
     * List files from FTP server
     * @return
     * @throws IOException
     */
    public List<String> list() throws IOException {

        FTPClient client = null;
        List<String> fileList = new ArrayList<String>();
        boolean listAllFiles = false;
        boolean isFirstTry = true;
        int numberOfTries = info.getAttempt();

        while (isFirstTry || (!listAllFiles && numberOfTries > 0)) {
            try {
                if (!isFirstTry)
                    Thread.sleep(info.getDelay());
                numberOfTries --;
                isFirstTry = false;
                client = connectToFTPDirectory();
                FTPFile[] files = client.listFiles();
                for (FTPFile file : files) {
                    if (file.isFile()) {
                        logger.finest("file : " + file.getName());
                        fileList.add(file.getName());
                    }
                }
                listAllFiles = true;
            } catch (IOException e) {
            } catch (InterruptedException e) {
            } 
        }
        if (!listAllFiles)
            throw new IOException("Can't list files from the FTP server");
            
        if (client != null) {
            logger.finest("disconnect FTP connection to " + info.getServer());
            client.logout();
            client.disconnect();
        }

        return fileList;
    }

    /**
     * Create a FTPClient using the current FTPConnectionInfo, in the defined
     * Directory. Set active/passive mode and auto/ascii/bin transfer type
     * 
     * @return
     * @throws IOException
     */
    private FTPClient connectToFTPDirectory() throws IOException {
        logger.finest("create FTP connection to " + info.getServer());
        FTPClient client = FTPClientPool.createNewFTPClient(info.getServer(), info.getPort(), info
                .getUser(), info.getPassword());

        if (info.getDirectory().length() > 0) {
            client.changeWorkingDirectory(info.getDirectory());
        }
        // connection mode
        if (info.isPassiveMode()) {
            client.enterLocalPassiveMode();
            logger.finest("enter Passive mode");
        }
        // transfer mode
        if (info.isAsciiTransferType()) {
            client.setFileType(FTP.ASCII_FILE_TYPE);
            logger.finest("ascii transfer mode enable");
        } else if (info.isBinaryTransferType()) {
            client.setFileType(FTP.BINARY_FILE_TYPE);
            logger.finest("binary transfer mode enable");
        }

        logger.finest(info.toString());
        logger.finest("FTP server working directory is : " + client.printWorkingDirectory());
        return client;

    }
}
